@@ -41,12 +41,14 @@ import functools
 import collections
 
 import click
-
 import gevent
 
 from . import core
 
 
+__version__ = "0.10.0"
+
+
 class error(core.ErrorHandler):
     def __init__(self, verbose=False):
         echo = functools.partial(click.echo, err=True)
@@ -115,7 +117,7 @@ verbose = click.option(
 
 color = click.option(
     "--color/--no-color",
-    default=True,
+    default=None,
     show_default=True,
     is_flag=True,
     help="colored output",
@@ -234,34 +236,9 @@ dry_run = click.option(
 )
 
 
-class ClickOutputHandler(logging.Handler):
-
-    Style = collections.defaultdict(dict)
-    Style.update(
-        {
-            logging.DEBUG: dict(dim=True),
-            logging.INFO: dict(),
-            logging.WARNING: dict(fg="yellow"),
-            logging.ERROR: dict(fg="red"),
-            logging.CRITICAL: dict(fg="magenta"),
-        }
-    )
-
-    def emit(self, record):
-        try:
-            record.name = record.name.replace("tangoctl.", "")
-            record.msg = click.style(record.msg, **self.Style[record.levelno])
-            click.echo(self.format(record))
-        except Exception:
-            self.handleError(record)
-
-
-def _prepare_logging():
+def prepare_logging(level=None):
     fmt = "%(asctime)s %(name)14s: %(message)s"
-    handler = ClickOutputHandler()
-    handler.addFilter(lambda record: record.name.startswith("tangoctl"))
-    handler.setLevel(logging.INFO)
-    logging.basicConfig(level=logging.INFO, format=fmt, handlers=[handler])
+    logging.basicConfig(level=level, format=fmt)
 
 
 def _ls_columns(seq, nb_cols=4):
@@ -378,6 +355,7 @@ def _attribute_info_table(device, attribute):
 
 @click.group(invoke_without_command=True)
 @color
+@click.version_option(version=__version__)
 @click.pass_context
 def cli(ctx, color):
     """Query or send control commands to the tango system.
@@ -404,12 +382,11 @@ def cli(ctx, color):
     $ tangoctl device attribute list sys/tg_test/1
 
     """
+    prepare_logging()
     ctx.color = color
     ctx.info_name = "tangoctl"
-    _prepare_logging()
     if ctx.invoked_subcommand is None:
         from . import repl
-
         repl.run(ctx)
 
 
@@ -522,8 +499,7 @@ def device_table(device, class_, server, host, exclude_dserver):
 @host_filter
 @verbose
 @dserver
-@click.pass_context
-def device_tree(ctx, device, class_, server, host, verbose, exclude_dserver):
+def device_tree(device, class_, server, host, verbose, exclude_dserver):
     """
     Show a tree of devices.
 
@@ -1259,58 +1235,6 @@ def starter_tree(starter, server, server_type, server_instance, all):
     click.echo(str(tree))
 
 
-def _starter_stop(
-    starter, server, server_type, server_instance, timeout, stop_timeout, dry_run
-):
-    if dry_run:
-        click.secho("Making a dry run!", bold=True, fg="red")
-    try:
-        core.starter_stop_servers(
-            starter_name=starter,
-            server_name=server,
-            server_type=server_type,
-            server_instance=server_instance,
-            timeout=timeout,
-            stop_timeout=stop_timeout,
-            dry_run=dry_run,
-        )
-    except gevent.Timeout:
-        click.secho("Operation timeout ({} s)".format(timeout), fg="yellow")
-
-
-def _starter_start(starter, server, server_type, server_instance, timeout, dry_run):
-    if dry_run:
-        click.secho("Making a dry run!", bold=True, fg="red")
-    try:
-        core.starter_start_servers(
-            starter_name=starter,
-            server_name=server,
-            server_type=server_type,
-            server_instance=server_instance,
-            timeout=timeout,
-            dry_run=dry_run,
-        )
-    except gevent.Timeout:
-        click.secho("Operation timeout ({} s)".format(timeout), fg="yellow")
-
-
-def _starter_restart(starter, server, server_type, server_instance, timeout, dry_run):
-    # TODO: if no filter provided ask if really want to do it
-    if dry_run:
-        click.secho("Making a dry run!", bold=True, fg="red")
-    try:
-        core.starter_restart_servers(
-            starter_name=starter,
-            server_name=server,
-            server_type=server_type,
-            server_instance=server_instance,
-            timeout=timeout,
-            dry_run=dry_run,
-        )
-    except gevent.Timeout:
-        click.secho("Operation timeout ({} s)".format(timeout), fg="yellow")
-
-
 @starter_group.command("stop")
 @starter_filter
 @server_filter
@@ -1333,8 +1257,26 @@ def starter_stop(
     (level-1) servers are stopped only when all higher level servers
     have been stopped.
     """
-    return _starter_stop(
-        starter, server, server_type, server_instance, timeout, stop_timeout, dry_run
+    def output(message):
+        level = message["level"]
+        fg = None
+        if level == "warn":
+            fg = "yellow"
+        elif level == "error":
+            fg = "red"
+        click.secho(message["text"], fg=fg)
+
+    if dry_run:
+        click.secho("Making a dry run!", bold=True, fg="red")
+    core.starter_stop_servers(
+        starter_name=starter,
+        server_name=server,
+        server_type=server_type,
+        server_instance=server_instance,
+        timeout=timeout,
+        stop_timeout=stop_timeout,
+        dry_run=dry_run,
+        output=output
     )
 
 
@@ -1357,8 +1299,26 @@ def starter_start(starter, server, server_type, server_instance, timeout, dry_ru
     (level+1) servers are started only when all lower level servers
     have been started.
     """
-    return _starter_start(
-        starter, server, server_type, server_instance, timeout, dry_run
+    def output(message):
+        level = message["level"]
+        fg = None
+        if level == "warn":
+            fg = "yellow"
+        elif level == "error":
+            fg = "red"
+        click.secho(message["text"], fg=fg)
+
+
+    if dry_run:
+        click.secho("Making a dry run!", bold=True, fg="red")
+    core.starter_start_servers(
+        starter_name=starter,
+        server_name=server,
+        server_type=server_type,
+        server_instance=server_instance,
+        timeout=timeout,
+        dry_run=dry_run,
+        output=output
     )
 
 
@@ -1368,8 +1328,9 @@ def starter_start(starter, server, server_type, server_instance, timeout, dry_ru
 @server_type_filter
 @server_instance_filter
 @timeout
+@stop_timeout
 @dry_run
-def starter_restart(starter, server, server_type, server_instance, timeout, dry_run):
+def starter_restart(starter, server, server_type, server_instance, timeout, stop_timeout, dry_run):
     """Restart selected servers.
 
     The process of starting selected servers only starts after all
@@ -1397,10 +1358,58 @@ def starter_restart(starter, server, server_type, server_instance, timeout, dry_
         (level+1) servers are started only when all lower level servers
         have been started.
     """
-    return _starter_restart(
-        starter, server, server_type, server_instance, timeout, dry_run
+    def output(message):
+        level = message["level"]
+        fg = None
+        if level == "warn":
+            fg = "yellow"
+        elif level == "error":
+            fg = "red"
+        click.secho(message["text"], fg=fg)
+
+
+    if dry_run:
+        click.secho("Making a dry run!", bold=True, fg="red")
+    core.starter_restart_servers(
+        starter_name=starter,
+        server_name=server,
+        server_type=server_type,
+        server_instance=server_instance,
+        timeout=timeout,
+        stop_timeout=stop_timeout,
+        dry_run=dry_run,
+        output=output
     )
 
 
-if __name__ == "__main__":
+@starter_group.command("log")
+@starter_filter
+def starter_log(starter):
+    def starter_log(starter):
+        try:
+            result = starter.log()
+        except Exception as error:
+            result = error
+        return starter, result
+
+    starters = list(core.iter_starters(starter_name=starter))
+    tasks = [gevent.spawn(starter_log, starter) for starter in starters]
+    with gevent.iwait(tasks) as task_iter:
+        for task in task_iter:
+            starter, log = task.value
+            click.echo(starter.name)
+            if isinstance(log, Exception):
+                from tango import DevFailed
+                if isinstance(log, DevFailed):
+                    log = log.args[0].desc
+                click.echo("  Error: {!r}".format(log), err=True)
+            else:
+                click.echo("\n".join("  " + line for line in log.splitlines()))
+
+
+def main():
     cli()
+
+
+if __name__ == "__main__":
+    main()
