@@ -14,6 +14,7 @@ import functools
 import collections
 
 import gevent
+import gevent.queue
 import tango.gevent
 
 import click
@@ -818,6 +819,18 @@ class Starter:
     def __contains__(self, name):
         return name in self.servers
 
+    def _info(self, message, *args):
+        text = message % args
+        return dict(level="info", text=text)
+
+    def _warn(self, message, *args):
+        text = message % args
+        return dict(level="warn", text=text)
+
+    def _error(self, message, *args):
+        text = message % args
+        return dict(level="error", text=text)
+
     def wait_for_state(self, name, cond):
         while True:
             state = self[name]["state"]
@@ -826,13 +839,16 @@ class Starter:
             gevent.sleep(0.2)
 
     def kill_server(self, name, dry_run=False):
+        return tuple(self.iter_kill_server(name, dry_run=dry_run))
+
+    def iter_kill_server(self, name, dry_run=False):
         state = self[name]["state"]
         if state == "FAULT":
-            self._log.info(
+            yield self._info(
                 "skipped killing %r (%s)", name, human_starter_server_state(state)
             )
             return True
-        self._log.info("killing %r...", name)
+        yield self._info("killing %r...", name)
         if not dry_run:
             try:
                 self.starter_device.HardKillServer(name)
@@ -840,99 +856,112 @@ class Starter:
                 err = err.args[0].desc
                 state = self[name]["state"]
                 if state == "FAULT":
-                    self._log.warning(
+                    yield self._warn(
                         "server %r was killed but reported: %s", name, err
                     )
                     return True
                 else:
-                    self._log.error("failed to kill %r: %s", name, err)
+                    self._error("failed to kill %r: %s", name, err)
                     return False
             state = self.wait_for_state(name, lambda s: s in ("MOVING", "FAULT"))
             if not state:
-                self._log.warn("failed to wait to kill %r", name)
+                yield self._warn("failed to wait to kill %r", name)
                 return False
             if state == "FAULT":
-                self._log.info("killed %r", name)
+                yield self._info("killed %r", name)
                 return True
             state = self.wait_for_state(name, lambda s: s == "FAULT")
             if not state:
-                self._log.warn("failed to kill %r", name)
+                yield self._error("failed to kill %r", name)
                 return False
-        self._log.info("killed %r", name)
+        yield self._info("killed %r", name)
         return True
 
     def stop_server(self, name, dry_run=False):
+        return tuple(self.iter_stop_server(name, dry_run=dry_run))
+
+    def iter_stop_server(self, name, dry_run=False):
         server = self[name]
-        state, level = server["state"], server["level"]
+        state = server["state"]
         if state == "FAULT":
-            self._log.info(
-                "skipped stopping %r (%s) (level %d)",
+            yield self._info(
+                "skipped stopping %r (%s)",
                 name,
                 human_starter_server_state(state),
-                level,
             )
             return True
-        self._log.info("stopping %r (level %d)...", name, level)
+        yield self._info("stopping %r...", name)
         if not dry_run:
             try:
                 self.starter_device.DevStop(name)
             except tango.DevFailed as err:
                 err = err.args[0].desc
-                self._log.warning("failed to stop on %r (%r)", name, err)
+                yield self._warn("failed to stop on %r (%r)", name, err)
                 return False
             state = self.wait_for_state(name, lambda s: s in ("MOVING", "FAULT"))
             if not state:
-                self._log.warn("failed to wait to stop %r", name)
+                yield self._warn("failed to wait to stop %r", name)
                 return False
             if state == "FAULT":
-                self._log.info("stopped %r", name)
+                yield self._info("stopped %r", name)
                 return True
             state = self.wait_for_state(name, lambda s: s == "FAULT")
             if not state:
-                self._log.warn("failed to stop %r", name)
+                yield self._warn("failed to stop %r", name)
                 return False
-        self._log.info("stopped %r", name)
+        yield self._info("stopped %r", name)
         return True
 
     def stop_kill_server(self, name, timeout=5, dry_run=False):
-        task = gevent.spawn(self.stop_server, name, dry_run=dry_run)
-        stopped = False
+        return tuple(self.iter_stop_kill_server(name, timeout=timeout, dry_run=dry_run))
+
+    def iter_stop_kill_server(self, name, timeout=5, dry_run=False):
         try:
-            stopped = task.get(timeout=timeout)
+            with gevent.Timeout(timeout):
+                yield from self.iter_stop_server(name, dry_run=dry_run)
         except gevent.Timeout:
-            self._log.warning("timeout trying to stop %s. Performing a kill", name)
-            stopped = False
-            task.kill()
-        if not stopped:
-            self.kill_server(name, dry_run=dry_run)
+            yield self._warn("timeout trying to stop %s. Performing a kill", name)
+            yield from self.iter_kill_server(name, dry_run=dry_run)
 
     def start_server(self, name, dry_run=False):
+        return tuple(self.iter_start_server(name, dry_run=dry_run))
+
+    def iter_start_server(self, name, dry_run=False):
         state = self[name]["state"]
         if state == "ON":
-            self._log.info(
+            yield self._info(
                 "skipped starting %r (%s)", name, human_starter_server_state(state)
             )
             return True
-        self._log.info("starting %r", name)
+        yield self._info("starting %r...", name)
         if not dry_run:
             self.starter_device.DevStart(name)
             state = self.wait_for_state(name, lambda s: s in ("MOVING", "ON"))
             if not state:
-                self._log.warn("failed to start %r", name)
+                yield self._warn("failed to start %r", name)
                 return False
             if state == "ON":
-                self._log.info("started %r", name)
+                yield self._info("started %r", name)
                 return True
             state = self.wait_for_state(name, lambda s: s == "ON")
             if not state:
-                self._log.warn("failed to start %r", name)
+                yield self._warn("failed to start %r", name)
                 return False
-        self._log.info("started %r", name)
+        yield self._info("started %r", name)
         return True
 
     def restart_server(self, name):
-        self.stop_server(name)
-        self.start_server(name)
+        return tuple(self.iter_restart_server(name))
+
+    def iter_restart_server(self, name):
+        yield from self.iter_stop_kill_server(name)
+        yield from self.iter_start_server(name)
+
+    def log(self):
+        return self.starter_device.command_inout("DevReadLog", "Starter")
+
+    def server_log(self, server):
+        return self.starter_device.command_inout("DevReadLog", "server")
 
 
 STARTERS = {}
@@ -1054,10 +1083,11 @@ def stop_server(name, db=None):
     starter = find_starter_for_server(name, db=db)
     if starter is None:
         next(server_stop([name]))
-    starter.stop_server(name)
+    else:
+        starter.stop_server(name)
 
 
-def starter_stop_servers(
+def iter_starter_stop_servers(
     starter_name=None,
     server_name=None,
     server_type=None,
@@ -1067,6 +1097,13 @@ def starter_stop_servers(
     dry_run=False,
     db=None,
 ):
+    def stop(starter, server, channel):
+        gen = starter.iter_stop_kill_server(
+            server, timeout=stop_timeout, dry_run=dry_run)
+        for message in gen:
+            channel.put(message)
+        # Tell listener we finished
+        channel.put(StopIteration)
 
     with gevent.Timeout(timeout):
         levels = find_starters_levels_for_filtered_servers(
@@ -1079,21 +1116,26 @@ def starter_stop_servers(
         )
         for level in sorted(levels, reverse=True):
             level_tasks = []
+            channel = gevent.queue.Queue()
+            yield {
+                "level": "info",
+                "text": "beginning to stop servers at level {}".format(level)
+            }
             for starter, servers in levels[level].items():
-                stop = starter.stop_kill_server
                 tasks = [
-                    gevent.spawn(
-                        stop, server["name"], timeout=stop_timeout, dry_run=dry_run
-                    )
+                    gevent.spawn(stop, starter, server["name"], channel)
                     for server in servers
                 ]
                 level_tasks.extend(tasks)
-            log.info("beginning to stop servers at level {}".format(level))
+            yield from channel
             gevent.joinall(level_tasks)
-            log.info("finished stopping level {}".format(level))
+            yield {
+                "level": "info",
+                "text": "finished stopping level {}".format(level)
+            }
 
 
-def starter_start_servers(
+def iter_starter_start_servers(
     starter_name=None,
     server_name=None,
     server_type=None,
@@ -1102,6 +1144,12 @@ def starter_start_servers(
     dry_run=False,
     db=None,
 ):
+    def start(starter, server, channel):
+        gen = starter.iter_start_server(server, dry_run=dry_run)
+        for message in gen:
+            channel.put(message)
+        # Tell listener we finished
+        channel.put(StopIteration)
 
     with gevent.Timeout(timeout):
         levels = find_starters_levels_for_filtered_servers(
@@ -1114,16 +1162,105 @@ def starter_start_servers(
         )
         for level in sorted(levels):
             level_tasks = []
+            channel = gevent.queue.Queue()
+            yield {
+                "level": "info",
+                "text": "beginning to start servers at level {}".format(level)
+            }
             for starter, servers in levels[level].items():
-                stop = starter.start_server
                 tasks = [
-                    gevent.spawn(stop, server["name"], dry_run=dry_run)
+                    gevent.spawn(start, starter, server["name"], channel)
                     for server in servers
                 ]
                 level_tasks.extend(tasks)
-            log.info("beginning to start servers at level {}".format(level))
+            yield from channel
             gevent.joinall(level_tasks)
-            log.info("finished starting servers at level {}".format(level))
+            yield {
+                "level": "info",
+                "text": "finished starting level {}".format(level)
+            }
+
+
+def iter_starter_restart_servers(
+    starter_name=None,
+    server_name=None,
+    server_type=None,
+    server_instance=None,
+    timeout=None,
+    stop_timeout=5,
+    dry_run=False,
+    db=None,
+):
+    db = get_db(db)
+
+    with gevent.Timeout(timeout):
+        yield from iter_starter_stop_servers(
+            starter_name=starter_name,
+            server_name=server_name,
+            server_type=server_type,
+            server_instance=server_instance,
+            stop_timeout=stop_timeout,
+            dry_run=dry_run,
+            db=db,
+        )
+        yield from iter_starter_start_servers(
+            starter_name=starter_name,
+            server_name=server_name,
+            server_type=server_type,
+            server_instance=server_instance,
+            dry_run=dry_run,
+            db=db,
+        )
+
+
+def starter_stop_servers(
+    starter_name=None,
+    server_name=None,
+    server_type=None,
+    server_instance=None,
+    timeout=None,
+    stop_timeout=5,
+    dry_run=False,
+    output=lambda m: print(m["text"]),
+    db=None,
+):
+    if output is None:
+        output = lambda x: None
+    steps = iter_starter_stop_servers(
+        starter_name=starter_name,
+        server_name=server_name,
+        server_type=server_type,
+        server_instance=server_instance,
+        timeout=timeout,
+        stop_timeout=stop_timeout,
+        dry_run=dry_run,
+        db=db)
+    for message in steps:
+        output(message)
+
+
+def starter_start_servers(
+    starter_name=None,
+    server_name=None,
+    server_type=None,
+    server_instance=None,
+    timeout=None,
+    dry_run=False,
+    output=lambda m: print(m["text"]),
+    db=None,
+):
+    if output is None:
+        output = lambda x: None
+    steps = iter_starter_start_servers(
+        starter_name=starter_name,
+        server_name=server_name,
+        server_type=server_type,
+        server_instance=server_instance,
+        timeout=timeout,
+        dry_run=dry_run,
+        db=db)
+    for message in steps:
+        output(message)
 
 
 def starter_restart_servers(
@@ -1132,29 +1269,23 @@ def starter_restart_servers(
     server_type=None,
     server_instance=None,
     timeout=None,
+    stop_timeout=5,
     dry_run=False,
+    output=lambda m: print(m["text"]),
     db=None,
 ):
-
-    db = get_db(db)
-
-    with gevent.Timeout(timeout):
-        starter_stop_servers(
-            starter_name=starter_name,
-            server_name=server_name,
-            server_type=server_type,
-            server_instance=server_instance,
-            dry_run=dry_run,
-            db=db,
-        )
-        starter_start_servers(
-            starter_name=starter_name,
-            server_name=server_name,
-            server_type=server_type,
-            server_instance=server_instance,
-            dry_run=dry_run,
-            db=db,
-        )
+    if output is None:
+        output = lambda x: None
+    steps = iter_starter_restart_servers(
+        starter_name=starter_name,
+        server_name=server_name,
+        server_type=server_type,
+        server_instance=server_instance,
+        timeout=timeout,
+        dry_run=dry_run,
+        db=db)
+    for message in steps:
+        output(message)
 
 
 def starter_tree(
